@@ -1,23 +1,23 @@
 /* -*- c-file-style: "linux" -*-
-   
-   Copyright (C) 1996-2001 by Andrew Tridgell 
-   Copyright (C) Paul Mackerras 1996
-   Copyright (C) 2001, 2002 by Martin Pool <mbp@samba.org>
-   
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-   
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-   
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+ * 
+ * Copyright (C) 1996-2001 by Andrew Tridgell 
+ * Copyright (C) Paul Mackerras 1996
+ * Copyright (C) 2001, 2002 by Martin Pool <mbp@samba.org>
+ * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
 
 /**
  * @file io.c
@@ -36,7 +36,7 @@
 
 #include "rsync.h"
 
-/* if no timeout is specified then use a 60 second select timeout */
+/** If no timeout is specified then use a 60 second select timeout */
 #define SELECT_TIMEOUT 60
 
 static int io_multiplexing_out;
@@ -52,6 +52,24 @@ extern int io_timeout;
 extern struct stats stats;
 
 
+const char phase_unknown[] = "unknown";
+
+/**
+ * The connection might be dropped at some point; perhaps because the
+ * remote instance crashed.  Just giving the offset on the stream is
+ * not very helpful.  So instead we try to make io_phase_name point to
+ * something useful.
+ *
+ * For buffered/multiplexed IO these names will be somewhat
+ * approximate; perhaps for ease of support we would rather make the
+ * buffer always flush when a single application-level IO finishes.
+ *
+ * @todo Perhaps we want some simple stack functionality, but there's
+ * no need to overdo it.
+ **/
+const char *io_write_phase = phase_unknown;
+const char *io_read_phase = phase_unknown;
+
 /** Ignore EOF errors while reading a module listing if the remote
     version is 24 or less. */
 int kludge_around_eof = False;
@@ -86,13 +104,13 @@ static void check_timeout(void)
 	}
 }
 
-/* setup the fd used to propogate errors */
+/** Setup the fd used to propagate errors */
 void io_set_error_fd(int fd)
 {
 	io_error_fd = fd;
 }
 
-/* read some data from the error fd and write it to the write log code */
+/** Read some data from the error fd and write it to the write log code */
 static void read_error_fd(void)
 {
 	char buf[200];
@@ -124,21 +142,18 @@ static void read_error_fd(void)
 }
 
 
+/**
+ * It's almost always an error to get an EOF when we're trying to read
+ * from the network, because the protocol is self-terminating.
+ *
+ * However, there is one unfortunate cases where it is not, which is
+ * rsync <2.4.6 sending a list of modules on a server, since the list
+ * is terminated by closing the socket. So, for the section of the
+ * program where that is a problem (start_socket_client),
+ * kludge_around_eof is True and we just exit.
+ */
 static void whine_about_eof (void)
 {
-	/**
-	   It's almost always an error to get an EOF when we're trying
-	   to read from the network, because the protocol is
-	   self-terminating.
-	   
-	   However, there is one unfortunate cases where it is not,
-	   which is rsync <2.4.6 sending a list of modules on a
-	   server, since the list is terminated by closing the socket.
-	   So, for the section of the program where that is a problem
-	   (start_socket_client), kludge_around_eof is True and we
-	   just exit.
-	*/
-
 	if (kludge_around_eof)
 		exit_cleanup (0);
 	else {
@@ -163,7 +178,7 @@ static void die_from_readerr (int err)
 }
 
 
-/*!
+/**
  * Read from a socket with IO timeout. return the number of bytes
  * read. If no bytes can be read then exit, never return a number <= 0.
  *
@@ -245,8 +260,10 @@ static int read_timeout (int fd, char *buf, size_t len)
 
 
 
-/*! Continue trying to read len bytes - don't return until len has
-  been read.   */
+/**
+ * Continue trying to read len bytes - don't return until len has been
+ * read.
+ **/
 static void read_loop (int fd, char *buf, size_t len)
 {
 	while (len) {
@@ -316,8 +333,11 @@ static int read_unbuffered(int fd, char *buf, size_t len)
 
 
 
-/* do a buffered read from fd. don't return until all N bytes
-   have been read. If all N can't be read then exit with an error */
+/**
+ * Do a buffered read from @p fd.  Don't return until all @p n bytes
+ * have been read.  If all @p n can't be read then exit with an
+ * error.
+ **/
 static void readfd (int fd, char *buffer, size_t N)
 {
 	int  ret;
@@ -387,8 +407,40 @@ unsigned char read_byte(int f)
 	return c;
 }
 
-/* Write len bytes to fd.  This underlies the multiplexing system,
- * which is always called by application code.  */
+
+/**
+ * Sleep after writing to limit I/O bandwidth usage.
+ *
+ * @todo Rather than sleeping after each write, it might be better to
+ * use some kind of averaging.  The current algorithm seems to always
+ * use a bit less bandwidth than specified, because it doesn't make up
+ * for slow periods.  But arguably this is a feature.  In addition, we
+ * ought to take the time used to write the data into account.
+ **/
+static void sleep_for_bwlimit(int bytes_written)
+{
+	struct timeval tv;
+
+	if (!bwlimit)
+		return;
+
+	assert(bytes_written > 0);
+	assert(bwlimit > 0);
+	
+	tv.tv_usec = bytes_written * 1000 / bwlimit;
+	tv.tv_sec  = tv.tv_usec / 1000000;
+	tv.tv_usec = tv.tv_usec % 1000000;
+
+	select(0, NULL, NULL, NULL, &tv);
+}
+
+
+/**
+ * Write len bytes to the file descriptor @p fd.
+ *
+ * This function underlies the multiplexing system.  The body of the
+ * application never calls this function directly.
+ **/
 static void writefd_unbuffered(int fd,char *buf,size_t len)
 {
 	size_t total = 0;
@@ -457,24 +509,13 @@ static void writefd_unbuffered(int fd,char *buf,size_t len)
 				 * across the stream */
 				io_multiplexing_close();
 				rprintf(FERROR, RSYNC_NAME
-					": error writing %d unbuffered bytes"
-					" - exiting: %s\n", len,
+					": writefd_unbuffered failed to write %ld bytes: phase \"%s\": %s\n",
+					(long) len, io_write_phase, 
 					strerror(errno));
 				exit_cleanup(RERR_STREAMIO);
 			}
 
-			/* Sleep after writing to limit I/O bandwidth */
-			if (bwlimit)
-			{
-			    tv.tv_sec = 0;
-			    tv.tv_usec = ret * 1000 / bwlimit;
-			    while (tv.tv_usec > 1000000)
-			    {
-				tv.tv_sec++;
-				tv.tv_usec -= 1000000;
-			    }
-			    select(0, NULL, NULL, NULL, &tv);
- 			}
+			sleep_for_bwlimit(ret);
  
 			total += ret;
 
@@ -499,8 +540,10 @@ void io_start_buffering(int fd)
 	io_buffer_count = 0;
 }
 
-/* write an message to a multiplexed stream. If this fails then rsync
-   exits */
+/**
+ * Write an message to a multiplexed stream. If this fails then rsync
+ * exits.
+ **/
 static void mplex_write(int fd, enum logcode code, char *buf, size_t len)
 {
 	char buffer[4096];
@@ -583,6 +626,14 @@ void write_int(int f,int32 x)
 }
 
 
+void write_int_named(int f, int32 x, const char *phase)
+{
+	io_write_phase = phase;
+	write_int(f, x);
+	io_write_phase = phase_unknown;
+}
+
+
 /*
  * Note: int64 may actually be a 32-bit type if ./configure couldn't find any
  * 64-bit types on this platform.
@@ -609,7 +660,7 @@ void write_buf(int f,char *buf,size_t len)
 	writefd(f,buf,len);
 }
 
-/* write a string to the connection */
+/** Write a string to the connection */
 static void write_sbuf(int f,char *buf)
 {
 	write_buf(f, buf, strlen(buf));
@@ -670,7 +721,7 @@ void io_printf(int fd, const char *format, ...)
 }
 
 
-/* setup for multiplexing an error stream with the data stream */
+/** Setup for multiplexing an error stream with the data stream */
 void io_start_multiplex_out(int fd)
 {
 	multiplex_out_fd = fd;
@@ -679,7 +730,7 @@ void io_start_multiplex_out(int fd)
 	io_multiplexing_out = 1;
 }
 
-/* setup for multiplexing an error stream with the data stream */
+/** Setup for multiplexing an error stream with the data stream */
 void io_start_multiplex_in(int fd)
 {
 	multiplex_in_fd = fd;
@@ -687,7 +738,7 @@ void io_start_multiplex_in(int fd)
 	io_multiplexing_in = 1;
 }
 
-/* write an message to the multiplexed error stream */
+/** Write an message to the multiplexed error stream */
 int io_multiplex_write(enum logcode code, char *buf, size_t len)
 {
 	if (!io_multiplexing_out) return 0;
@@ -698,7 +749,7 @@ int io_multiplex_write(enum logcode code, char *buf, size_t len)
 	return 1;
 }
 
-/* stop output multiplexing */
+/** Stop output multiplexing */
 void io_multiplexing_close(void)
 {
 	io_multiplexing_out = 0;
