@@ -1,6 +1,6 @@
 /* -*- c-file-style: "linux" -*-
  *
- * Copyright (C) 2001 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 2001, 2002 by Martin Pool <mbp@samba.org>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License version
@@ -17,10 +17,9 @@
  */
 
 /**
- * \section tls
+ * @file tls.c
  *
- * tls -- Trivial recursive ls, for comparing two directories after
- * running an rsync.
+ * Trivial @c ls for comparing two directories after running an rsync.
  *
  * The problem with using the system's own ls is that some features
  * have little quirks that make directories look different when for
@@ -39,7 +38,6 @@
  **/
 
 
-
 #include "rsync.h"
 
 #define PROGRAM "tls"
@@ -48,6 +46,7 @@
 int dry_run = 0;
 int read_only = 1;
 int list_only = 0;
+int preserve_perms = 0;
 
 
 static void failed (char const *what,
@@ -62,7 +61,7 @@ static void failed (char const *what,
 
 static void list_file (const char *fname)
 {
-	struct stat buf;
+	STRUCT_STAT buf;
 	char permbuf[PERMSTRING_SIZE];
 	struct tm *mt;
 	char datebuf[50];
@@ -117,9 +116,10 @@ static void list_file (const char *fname)
 	
 	/* NB: need to pass size as a double because it might be be
 	 * too large for a long. */
-	printf("%s %12.0f %6d.%-6d %s %s%s\n",
+	printf("%s %12.0f %6ld.%-6ld %6d %s %s%s\n",
 	       permbuf, (double) buf.st_size,
-	       buf.st_uid, buf.st_gid,
+	       (long) buf.st_uid, (long) buf.st_gid,
+	       buf.st_nlink,
 	       datebuf, fname, linkbuf);
 }
 
